#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>
#include "SDL.h"
#include "bitmap.h"

#include "../gpf.c"



/* screen width, height, and bit depth */
#define SCREEN_WIDTH  128
#define SCREEN_HEIGHT 128
#define SCREEN_BPP     16
#define font_height   12
#define font_width    7

/* Set up some booleans */
#define TRUE  1
#define FALSE 0

/* This is our SDL surface */
SDL_Surface *surface;

GLuint  base; /* Base Display List For The Font Set */
GLfloat cnt1; /* 1st Counter Used To Move Text & For Coloring */
GLfloat cnt2; /* 2nd Counter Used To Move Text & For Coloring */

/* function to recover memory form our list of characters */
GLvoid KillFont( GLvoid )
{
	glDeleteLists( base, 96 );
	return;
}

/* function to release/destroy our resources and restoring the old desktop */
void Quit( int returnCode )
{

	KillFont( );
	/* clean up the window */
	SDL_Quit( );

	/* and exit appropriately */
	exit( returnCode );
}

/* function to build our font list */
GLvoid buildFont( GLvoid )
{
    Display *dpy;          /* Our current X display */
    XFontStruct *fontInfo; /* Our font info */

    /* Sotrage for 96 characters */
    base = glGenLists( 96 );

    /* Get our current display long enough to get the fonts */
    dpy = XOpenDisplay( NULL );

    /* Get the font information */
    /* fontInfo = XLoadQueryFont( dpy, "-adobe-helvetica-medium-r-normal--18-*-*-*-p-*-iso8859-1" ); */
    fontInfo = XLoadQueryFont( dpy, "-b&h-lucidatypewriter-medium-r-normal-*-12-*-*-*-m-*-iso8859-1");

    /* If the above font didn't exist try one that should */
    if ( fontInfo == NULL )
	{
	    fontInfo = XLoadQueryFont( dpy, "fixed" );
	    /* If that font doesn't exist, something is wrong */
	    if ( fontInfo == NULL )
		{
		    fprintf( stderr, "no X font available?\n" );
		    Quit( 1 );
		}
	}
    /* generate the list */
    glXUseXFont( fontInfo->fid, 32, 96, base );
    /* Recover some memory */
    XFreeFont( dpy, fontInfo );
    /* close the display now that we're done with it */
    XCloseDisplay( dpy );

    return;
}

/* Print our GL text to the screen */
GLvoid glPrint( const char *fmt, ... )
{
    char text[256]; /* Holds our string */
    va_list ap;     /* Pointer to our list of elements */

    /* If there's no text, do nothing */
    if ( fmt == NULL )
	return;

    /* Parses The String For Variables */
    va_start( ap, fmt );
      /* Converts Symbols To Actual Numbers */
      vsprintf( text, fmt, ap );
    va_end( ap );
    
    /* Pushes the Display List Bits */
    glPushAttrib( GL_LIST_BIT );

    /* Sets base character to 32 */
//    glListBase( base - 32 );
    glListBase( base );

    /* Draws the text */
    glCallLists( strlen( text ), GL_UNSIGNED_BYTE, text );

    /* Pops the Display List Bits */
    glPopAttrib( );
}


GLvoid DrawChar(char ch)
{
	//    glRasterPos2i( x ,SCREEN_HEIGHT - y  - font_height);

	/* If there's no text, do nothing */
	if ( ch == 0 )
		return;
	/* Pushes the Display List Bits */
	glPushAttrib( GL_LIST_BIT );
	/* Sets base character to 32 */
	//    glListBase( base - 32 );
	glListBase( base );
	/* Draws the text */
	glCallLists( 1, GL_UNSIGNED_BYTE, &ch );
	/* Pops the Display List Bits */
	glPopAttrib( );
}

GLvoid DrawCharXY(int x,int y,char ch)
{
	glRasterPos2i( x ,SCREEN_HEIGHT - y  - font_height);

	if ( ch == 0 )
		return;
	glPushAttrib( GL_LIST_BIT );
	glListBase( base - 32 );
	glCallLists( 1, GL_UNSIGNED_BYTE, &ch );
	glPopAttrib( );
}



/* function to reset our viewport after a window resize */
int resizeWindow( int width, int height )
{
	GLfloat ratio;

	if ( height == 0 )
		height = 1;

	ratio = ( GLfloat )width / ( GLfloat )height;

	glViewport( 0, 0, ( GLint )width, ( GLint )height );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );


	glMatrixMode(GL_PROJECTION);                                                                 
	glLoadIdentity();                                                                            
	glOrtho(0,width, 0, height, -5.0, 5.0);  //   glOrtho(-0.5, 639.5, -0.5, 479.5, -1.0, 1.0);      
	glMatrixMode(GL_MODELVIEW);                                                                  
	glLoadIdentity();                                                                            


	return( TRUE );
}


/* general OpenGL initialization function */
int initGL( GLvoid )
{

	//DEL    glShadeModel( GL_SMOOTH );

	/* Set the background black */
	glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

	//    glClearDepth( 1.0f );

	/* Enables Depth Testing */
	//    glEnable( GL_DEPTH_TEST );

	/* The Type Of Depth Test To Do */
	//    glDepthFunc( GL_LEQUAL );

	/* Really Nice Perspective Calculations */
	//DEL    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );


	buildFont( );
    return( TRUE );
}

/* Here goes our drawing code */
int drawGLScene( GLvoid )
{
	/* These are to calculate our fps */
	static GLint T0     = 0;
	static GLint Frames = 0;
	int x,y,fw,fh;char c;
	//    char buffer[256];

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glLoadIdentity( );

	/* Move into the screen 1 unit */
	//    glTranslatef( 0.0f, 0.0f, -1.0f );

	/* Pulsing Colors Based On Text Position */
	glColor3f( 1.0f,1.0f,1.0f);

	//    for(y=0;y<16;y+=1)
	//	for(x=0;x<16;x+=1)
	//	{    glRasterPos2f( (float)x*12 ,(float)y*12);
	///	    DrawChar((char)y*16+x);
	//	}

	/*
	   for(y=0;y<12;y+=1)
	   {
	   glRasterPos2i( 0 ,SCREEN_HEIGHT-y*12);
	   for(x=0;x<16;x+=1)    
	   buffer[x]=y*16+x;
	   buffer[16]=0;  //	printf("%s\n",buffer);
	   glPrint(buffer);
	   }
	   */

	c=32;
	fw=SCREEN_WIDTH/16;
	fh=SCREEN_HEIGHT/8;

	for(y=0; y< 8; y++)
	{
		for(x=0;x<16;x++)
			DrawCharXY(x*fw,y*fh,c++);
	}
	// DrawCharXY(0,0,'a');
	//    DrawCharXY(0,12,'b');


	SDL_GL_SwapBuffers();

	return( TRUE );
}


int        Width;       /* Width of window */
int        Height;      /* Height of window */
BITMAPINFO *BitmapInfo; /* Bitmap information */
GLubyte    *BitmapBits; /* Bitmap data */


int SaveFontGPF()
{
	GPF *g;
	GLsizei width=SCREEN_HEIGHT,height=SCREEN_WIDTH;

	BitmapBits=malloc(width*height*3);
	glFinish();
	glReadPixels(0,0,width,height,GL_RGB,GL_UNSIGNED_BYTE,BitmapBits );

	g=New_GPF(width,height,GPF_RGB,BitmapBits);

	//    GPF_RGB2RGBA(g);
	GPF_filesave(g,"font.gpf");
	printf("Saved GPF!!\n");
	return 1;
}


void handleKeyPress( SDL_keysym *keysym )
{
	switch ( keysym->sym )
	{
		case SDLK_ESCAPE:
			Quit( 0 );
			break;
		case SDLK_F1:
			SaveFontGPF();
			break;
		default:
			break;
	}

	return;
}



int main( int argc, char **argv )
{

	int videoFlags;
	int done = FALSE;
	SDL_Event event;
	const SDL_VideoInfo *videoInfo;
	int isActive = TRUE;

	/* initialize SDL */
	if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
		fprintf( stderr, "Video initialization failed: %s\n",
				SDL_GetError( ) );
		Quit( 1 );
	}

	/* the flags to pass to SDL_SetVideoMode */
	videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
	videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
	videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
	//    videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */
	videoFlags |= SDL_HWSURFACE;
	//	videoFlags |= SDL_SWSURFACE;

	/* Sets up OpenGL double buffering */
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	/* get a SDL surface */
	surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
			videoFlags );

	/* Verify there is a surface */
	if ( !surface )
	{
		fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
		Quit( 1 );
	}

	initGL();

	resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

	/* wait for events */
	while ( !done )
	{
		/* handle the events in the queue */

		while ( SDL_PollEvent( &event ) )
		{
			switch( event.type )
			{
				case SDL_ACTIVEEVENT:
					/* Something's happend with our focus
					 * If we lost focus or we are iconified, we
					 * shouldn't draw the screen
					 */
					if ( event.active.gain == 0 )
						isActive = FALSE;
					else
						isActive = TRUE;
					break;
				case SDL_VIDEORESIZE:
					/* handle resize event */
					surface = SDL_SetVideoMode( event.resize.w,
							event.resize.h,
							16, videoFlags );
					if ( !surface )
					{
						fprintf( stderr, "Could not get a surface after resize: %s\n", SDL_GetError( ) );
						Quit( 1 );
					}
					resizeWindow( event.resize.w, event.resize.h );
					break;
				case SDL_KEYDOWN:
					/* handle key presses */
					handleKeyPress( &event.key.keysym );
					break;
				case SDL_QUIT:
					/* handle quit requests */
					done = TRUE;
					break;
				default:
					break;
			}
		}

		/* draw the scene */
		drawGLScene( );
	}

	Quit( 0 );

	return( 0 );
}
