/*
 * This code was created by Jeff Molofee '99 
 * (ported to Linux/SDL by Ti Leggett '01)
 *
 * If you've found this code useful, please let me know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 * 
 * or for port-specific comments, questions, bugreports etc. 
 * email to leggett@eecs.tulane.edu
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"
#include "SDL_image.h"
#include "../../gpf.c"


/* screen width, height, and bit depth */
#define SCREEN_WIDTH  640

#define SCREEN_HEIGHT 480
#define SCREEN_BPP     16
#define ScreenHeight SCREEN_HEIGHT

/* Set up some booleans */
#define TRUE  1
#define FALSE 0

SDL_Surface *surface; /* This is our SDL surface */

GLuint texture[2];    /* Storage For One Texture                           */

int scroll;           /* Used For Scrolling The Screen                     */
int maxtokens;        /* Keeps Track Of The Number Of Extensions Supported */
int swidth;           /* Scissor Width                                     */
int sheight;          /* Scissor Height                                    */

GLuint base;          /* Base Display List For The Font                    */

#include "libfont.c"

/* function to release/destroy our resources and restoring the old desktop */
void Quit( int returnCode )
{
    /* Delete All 256 Display Lists */
    glDeleteLists( base, 256 );
    /* clean up the window */
    SDL_Quit( );
    /* and exit appropriately */
    exit( returnCode );
}

SDL_Surface *TextureImage; 
//int font_image_height=0;
//int texture_font_lucida;

/* function to load in bitmap as a GL texture */


/* function to reset our viewport after a window resize */
int resizeWindow( int width, int height )
{
 
    if ( height == 0 )
	height = 1;

    swidth  = width;  /* Set scissor width to window width   */
    sheight = height; /* Set scissor height to window height */

    /* Setup our viewport. */
    glViewport( 0, 0, ( GLint )width, ( GLint )height );

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    /* Create Ortho 640x480 View */
//    glOrtho( 0, SCREEN_WIDTH, SCREEN_HEIGHT, 0.0f, -1.0f, 1.0f );
    glOrtho(0,width, 0, height, -5.0, 5.0);  
//   glOrtho(-0.5, 639.5, -0.5, 479.5, -1.0, 1.0);      

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity( );

    return( TRUE );
}

/* function to handle key press events */
void handleKeyPress( SDL_keysym *keysym )
{
    switch ( keysym->sym )
	{
	case SDLK_ESCAPE:
	    Quit( 0 );
	    break;
	case SDLK_F1:
	    /* F1 key was pressed
	     * this toggles fullscreen mode
	     */
	    SDL_WM_ToggleFullScreen( surface );
	    break;
	default:
	    break;
	}

    return;
}



/* Here goes our drawing code */
SDL_Event event;

int drawGLScene( GLvoid )
{
    /* These are to calculate our fps */
    static GLint T0     = 0;
    static GLint Frames = 0;

    glClearColor( 0.0f, 0.4f, 0.5f, 0.0f );
    /* Clear The Screen And The Depth Buffer */
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

//  glColor3f( 0.5f, 0.5f, 1.0f );

//  glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Select The Type Of Blending
//  glEnable(GL_BLEND);									// Enable Blending
  glColor3f(0.9,0.9,0.0);    
  DrawStrXY(0,0,"1234567890");
  DrawStrXY(0,15,"ABCDEabcde");
///    Draw_S();

    /*  Reset The ModelView Matrix */
    glLoadIdentity( ); 

///    printf("size.w=%d\n",event.resize.w);
    SDL_GL_SwapBuffers( );

    Frames++;
    {
	GLint t = SDL_GetTicks();
	if (t - T0 >= 5000) {
	    GLfloat seconds = (t - T0) / 1000.0;
	    GLfloat fps = Frames / seconds;
	    printf("%d frames in %g seconds = %g FPS\n", Frames, seconds, fps);
	    T0 = t;
	    Frames = 0;
	}
    }

    return( TRUE );
}

int main( int argc, char **argv )
{
    int videoFlags;
    int done = FALSE;
    const SDL_VideoInfo *videoInfo;
    int isActive = TRUE;
    int depth;

    /* initialize SDL */
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    fprintf( stderr, "Video initialization failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* Fetch the video info */
    videoInfo = SDL_GetVideoInfo( );

    if ( !videoInfo )
	{
	    fprintf( stderr, "Video query failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* the flags to pass to SDL_SetVideoMode */
    videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL          */
    videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering       */
    videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    videoFlags |= SDL_RESIZABLE;       /* Enable window resizing        */

    /* This checks to see if surfaces can be stored in memory */
    if ( videoInfo->hw_available )
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    /* This checks if hardware blits can be done */
    if ( videoInfo->blit_hw )
	videoFlags |= SDL_HWACCEL;

    /* Sets up OpenGL double buffering */
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    depth = SDL_VideoModeOK( SCREEN_WIDTH, SCREEN_HEIGHT, 32, videoFlags );
    printf( "Depth: %u\n", depth );
	    
    /* get a SDL surface */
    surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
				videoFlags );

    /* Verify there is a surface */
    if ( !surface )
	{
	    fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
	    Quit( 1 );
	}

    /* Enable key repeat */
    if ( ( SDL_EnableKeyRepeat( 100, SDL_DEFAULT_REPEAT_INTERVAL ) ) )
	{
	    fprintf( stderr, "Setting keyboard repeat failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* initialize OpenGL */


    InitLibFont( );

    /* resize the initial window */
    resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

    /* wait for events */
    while ( !done )
	{
	    /* handle the events in the queue */

	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			case SDL_ACTIVEEVENT:
			    /* Something's happend with our focus
			     * If we lost focus or we are iconified, we
			     * shouldn't draw the screen
			     */
			    if ( event.active.gain == 0 )
				isActive = FALSE;
			    else
				isActive = TRUE;
			    break;			    
			case SDL_VIDEORESIZE:
			    /* handle resize event */
			    surface = SDL_SetVideoMode( event.resize.w,
							event.resize.h,
							16, videoFlags );
			    if ( !surface )
				{
				    fprintf( stderr, "Could not get a surface after resize: %s\n", SDL_GetError( ) );
				    Quit( 1 );
				}
			    resizeWindow( event.resize.w, event.resize.h );
			    break;
			case SDL_KEYDOWN:
			    /* handle key presses */
			    handleKeyPress( &event.key.keysym );
			    break;
			case SDL_QUIT:
			    done = TRUE;
			    break;
			default:
			    break;
			}
		}
//	    if ( isActive )
		drawGLScene( );
	}

    return( 0 );
}
