#include "system.h"
#include "math.h"
#include "misc.h"
#include "ui/ui.h"
#include "ui/widgets.h"
#include "object/map.h"
#include "object/viewer.h"
#include "object/model.h"
#include "object/model_cs.h"

extern Object	*map_object;
// Example Object : show FPS count
class FPS : public Object 
{	
	private:
		int a_msec;
		int msec_prev;
		int count;
		int token;
		long t0;
	public:
	FPS():Object("FPS")
	{
		a_msec=0;
		count=0;
		msec_prev=0;
		token=debug_token();
		t0=current_time();
	}
	//old style 
	int Update(Uint32 msec){ 
		a_msec+=msec;
		if(a_msec>1000)
		{
			float sec=a_msec/1000;
			a_msec=a_msec%1000;
			float fps=count/sec;
			debug_printf(token,"fps=%3.0f",fps);
			count=0;
		}
		count++;
	}
	// new 	
	int Update2(Uint32 msec){ 
		long t=current_time();
		long usec=t-t0;
		//printf("usec=%ld \n", usec);
		if(usec >300000L)
		{
			float sec=usec/1000000.0;
			float fps=count/sec;

			debug_printf(token,"fps=%4.0f",fps);
			//printf("fps=%3.1f\n",fps);
			t0=t;
			count=0;
		}
		count++;
	}
};


// randomly eat CPU many time , for testing
class TimeSpoiler: public Object 
{	
	private:
		int token;
		int state;
	public:
	TimeSpoiler():Object("TimeSpoiler")
	{
		// 3% pobility!
		token=debug_token();
		state=0;		
	}
	int Update(Uint32 msec){ 
		int p=random()%100;
		if(p<3) 
		{
			//eat cpu
			//
			//do something

		}
	}
};

extern void depth_clear();
extern void depth_clear1();
extern void depth_clear2();

class Optimizer: public Object 
{	
	private:
		int token;
		int state;
		long t0;
	public:
	Optimizer():Object("Optimizer")
	{
		token=debug_token();
		state=0;		
		//state=1;		
		t0=current_time();
	}
	int Update(Uint32 msec){ 
		if(state) return 0;

		t0=current_time();
		for(int i=0;i<1000;i++)
			depth_clear();
		t0=current_time()-t0;
		console_printf(" depth_clear() = %ld ms",t0);

		t0=current_time();
		for(int i=0;i<1000;i++)
			depth_clear1();
		t0=current_time()-t0;
		console_printf(" depth_clear1() = %ld ms",t0);
		
		t0=current_time();
		t0=current_time()-t0;
		console_printf(" () = %ld ms",t0);
		state=1;
	}
};

//  putenv("SDL_VIDEODRIVER=dga");
struct route_s {
	float v[4];
	int x,y,z;
	char  val;
	int f,g,h;
	route_s *parent;
	route_s *next;
	char *pt;
	
} route;

class Point {
	public:
	int x,y;
	Point(int x0,int y0)
	{x=x0,y=y0;}
};
#include <stdlib.h>
// 20M ~40M
#define MAP_MAX 	128
#define	MAP_DEPTH 	1024*10
//  http://www.policyalmanac.org/games/aStarTutorial.htm
class PathFinder {
	char ***map;
	Index < route_s* > open_list;  
	Index < route_s* > closed_list;
	int max;
	route_s root;
	route_s target;

	void set(char ***p,int Max)
	{
		map=p;
		max=Max;
	}
	void clear()
	{
	}
	int  check(Index<route_s *> *l,char *a)
	{
		int size=l->size();
		for( int i=0;i<size;i++)
			if((*l)[i]->pt == a) 
				return 1;
		return 0; 
	}
	int  checkClose(route_s *a)
	{
		int size=closed_list.size();
		for( int i=0;i<size;i++)
			if(closed_list[i]->x == a->x 
					and closed_list[i]->y == a->y 
					and closed_list[i]->z == a->z)

				return 1;
		return 0; 
	}
	route_s* checkOpen(route_s *a)
	{
		int size=open_list.size();
		for( int i=0;i<size;i++)
			if(open_list[i]->x == a->x 
					and open_list[i]->y == a->y 
					and open_list[i]->z == a->z)
				return open_list[i];
		return 0; 
	}
	int getLowCostNodeIdx()
	{
		int low_f=99999;
		int idx=-1;
		route_s *N=NULL;
		int size=open_list.size();
		for( int i=0;i<size;i++)
			if(open_list[i]->f < low_f) 
			{
				low_f=open_list[i]->f;
				idx=i;
			}
		return idx; 
	}
	//void get_adjcent_list(int x,int y,int *n,char *list)
	int getDist(int x,int y,int z)
	{
		int k=abs(x) + abs(y) + abs(z);
		if (k==1) return 10;
		if (k==2) return 14;
		if (k==3) return 16;
		return 0;
	}
	int getH(route_s *N)
	{
		int k=abs(target.x-N->x)+abs(target.y-N->y)+abs(target.z-N->z);		
		return k*10;
	}

	int get_adjcent_list(route_s *N,route_s *list[])
	{
		int i=0;
		int px,py,pz;
		px=N->x;py=N->y;pz=N->z;
	
		if(px<=0 or py <=0) return 0 ;
		if(px>=max-1 or py>=max-1) return 0;

		int x,y,z;
		for(x=-1;x<2;x++)
		for(y=-1;y<2;y++)
		for(z=-1;z<2;z++)
		{
			route_s *n;
			if(x==0 and y==0 and z==0) continue;
			if(map[px+x][py+y][pz+z]==0)
			{
				n=new route_s();
				n->parent=N;
				n->f=N->g+getH(n);
				n->g=N->g+getDist(x,y,z);
				n->x=px+x;
				n->y=py+y;
				n->z=pz+z;
				list[i++]=n;
			}
		}
		return i;
	}
	
	int path()
	{
		open_list.add(&root);
		//path_loop();

	}

	int path_loop()
	{
		int x,y,n;
		char *target;
		route_s *adjs[32];
		route_s *N;
		
		while(1)
		{
			int idx;
			if(open_list.size()== 0) return 0; //fail, no path
			idx=getLowCostNodeIdx();
			N=open_list[idx];
			if(N->pt==target) return 1; // found 
		
			open_list.remove(idx);
			closed_list.add(N);

			n=get_adjcent_list(N,adjs);

		for(int i;i<n;i++)
		{
			route_s *r,*o;
			r=adjs[i];
			//1.
			if(checkClose(r)) continue;
			//2.
			o=checkOpen(r);
			if(o==0)
				open_list.add(r);// add_to
			else 
			{
				if ( r->g < o->g)
					r->parent=N;
			}
			//if(target==adjs[i]) 
			//	break;		
		}
		}
	}

};

class Heightmap {
	int rx,ry;
	int tx,ty;
	float ox,oy,oz;
	float offst_x,offst_y;
	float dx,dy,dz;
	Object *top_tree;
	char 	room[MAP_MAX][MAP_MAX][MAP_MAX];
	int 	tresh_h;
	int	index[256];
	int	null_n;
	public:
	GLuint TexId;
	Heightmap(){
		init();
	}
	void init()
	{
		null_n=MAP_MAX*MAP_MAX*MAP_MAX;
		top_tree=NULL;
		tresh_h=5;
		clear();
		glGenTextures (1,&TexId);
		glBindTexture (GL_TEXTURE_2D, TexId);
		
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		//this need OpenGL 2.0 more , not 2n size 
		//GL_ARB_texture_non_power_of_two
	//	glTexImage2D( GL_TEXTURE_2D, 0,GL_LUMINANCE, MAP_MAX, MAP_MAX
	//		,0, GL_LUMINANCE, GL_UNSIGNED_SHORT, map);

	}
	
	void set_top_tree(Object *obj)
	{
		top_tree=obj;

		int size=obj->children.size(); 
		for(int i=0;i<size;i++)
		{
			Object *c,*node;
			c=obj->children[i];
			//console_printf("");
			set_obstacle(c);
		}
	}

	void set_obstacle(Object *obj) // value = 255 
	{
		if(obj==NULL) { console_printf (" Object NULL!!");return; }
		if(obj->level>0) return ;

		int n=obj->triangles.size();
		int min[3],max[3];	
		for(int i=0;i<3;i++) 
		{
			min[i]=convert(i,obj->boundbox.min[i]);
			max[i]=convert(i,obj->boundbox.max[i]);
		}

		for(int x=min[0];x<max[0];x++)
		for(int y=min[1];y<max[1];y++)
		for(int z=min[2];z<max[2];z++)
			room[x][y][z]=255;
	}

	void clear()
	{
		int x,y,z;
		for(z=0;z<MAP_MAX;z++)
		for(y=0;y<MAP_MAX;y++)
		for(x=0;x<MAP_MAX;x++)
			room[z][y][x]=0;
	}
	void draw()
	{
		glMatrixMode (GL_PROJECTION);
		glLoadIdentity (); 		
		glMatrixMode (GL_MODELVIEW);
		glLoadIdentity (); 		

		int w=64,h=64;
		glEnable(GL_TEXTURE_2D);
  		glBindTexture(GL_TEXTURE_2D,TexId);
		glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(0,0);
		glTexCoord2f(1,0); glVertex2f(w,0);
		glTexCoord2f(1,1); glVertex2f(w,h);
		glTexCoord2f(0,1); glVertex2f(0,h);
		glEnd();
	}
	void recenter()
	{
		int i,j;
		for(i=0;i<MAP_MAX;i++)
		{
		
		}
	}

	int find_route(int dp[2],int sp[2])
	{
		
	}
	
	void convert(int x,int y,float fp[2])
	{
		float fx,fy;
		fx=dx*x - offst_x;
		fy=dy*y - offst_y;
		fp[0]=fx;
		fp[1]=fy;
	}

	int convert(int axis,float v)
	{
		int i;
		if(axis == 0) // x 
		{
			float x = v-ox;
			i=(int)( x/dx);  // dx==1.0 -> 6.5 , 0.5
		}
		else if(axis == 1) // y 
		{
			float y = v-oy;
			i=(int) (y/dy);  // dx==1.0 -> 6.5 , 0.5
		}
		else if(axis == 2) // z 
		{
			float z = v-oz;
			i=(int)(z/dz);  // dx==1.0 -> 6.5 , 0.5
		}
		return i;
	}

	void setlevel(float x,float y,float z)
	{
		
	}
	void setmap(Object *tree)
	{
		float minx,maxx,maxy,miny;
		//tree->boundbox.update();
		minx=tree->boundbox.min[0];
		miny=tree->boundbox.min[1];
		maxx=tree->boundbox.max[0];
		maxy=tree->boundbox.max[1];
		dx=(maxx-minx)/255.0;	
		dy=(maxy-miny)/255.0;
		offst_x=-minx;
		offst_y=-miny;
	}

};

Heightmap hmap;
void hmap_wraper()
{
	static int s=0;
//	console_printf("hmap=%d",s++);
//	hmap.draw();
}

class Navigator: public Object 
{	
	private:
		int a_msec;
		int count;
		int token;
		int state;
		float sx,sy;
		float rate;
		Frame *frame;
		Heightmap map;
	public:
	Navigator():Object("Navigator")
	{
		a_msec=0;
		count=0;
		token=debug_token();
		state=0;
		//void *(func)();
		//void (Heightmap::*fptr )( ) ;
		frame=new Frame(0,100,128,128,hmap_wraper);
		//frame=new Frame(100,200,128,128);
		system_add(frame); //frame->move(-220,20);
		//debug_printf(token,"Navigator!");
		console_printf("Navigator!");
	}
	
	void clear()
	{
	}
	
	void find_center(Object *o)
	{
		float vp[3],v1[3];		
		extern Object space;
		Line line;
		
		vectorCopy3f(line.onPoint,o->position);
		vectorCopy3f(line.normal,o->direction);

		Object *om=0;
		//BoundBox *box=&camera->boundbox;
		//debug_printf(token,"camera center(%f,%f,%f)",box->center[0],box->center[1],	box->center[2]);
	}


	void make_route()
	{

	}

	void request(Object *o,float target[3])
	{

	}

	void solve_route()
	{
	}

	void save()
	{
	}

	int Update(Uint32 msec){ 
		if(state==0)
		{
		}
		count++;
	}
};

// AI base Model 
#define STATE_FIND_TARGET 5
#define SEEK 6
#define LOOK_AROUND 6
#define MOVE 6
#define FIRE 6
#define REQUEST_PENDING 0x08

class Tracer : public Object 
{	
	private:
		int a_msec;
		int count;
		int token;
		int state;
		Model_CS *model;
	//	List< vector > route;
	public:
	Tracer(Object *obj):Object("Tracer")
	{
		a_msec=0;
		count=0;
		token=debug_token();
		state=0;
		model=(Model_CS*)obj;
	}
	
	void setModel(Object *obj)
	{
		model=(Model_CS*)obj;
	}

	int Update(Uint32 msec){ 
		//console_printf("aaaaaaaaaaa");
		//model->action(RUN);
		if(state==0)
		{
			state=SEEK;				
		}
		if(state==SEEK)
		{
			debug_printf(token," Thinking ");

		}
		count++;
	}
};

class Particle {
	float 	position[3];
	float 	force[3];
	int	life_time; // ms
}; 

// glTexSubImage2D
#define MASK_SIZE 256
class Particler : public Object 
{	
	private:
		int token;
		int w,h;
		int particle_n;
		Object *target;
		GLuint	TexId;
		List<Object*> list; // Index better !!!
		char mask[MASK_SIZE*MASK_SIZE];
		
		Index<Particle*> particles;
		int a;

	public:
	Particler():Object("Particler")
	{
		token=debug_token();
		target=0;
		particle_n=0;
		
		w=system_geti(SYSTEM_WIDTH);
		h=system_geti(SYSTEM_HEIGHT);
		
		glGenTextures (1,&TexId);
		glBindTexture (GL_TEXTURE_2D, TexId);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,GL_CLAMP);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,GL_CLAMP);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	
		//this need OpenGL 2.0 more , not 2n size 
		//GL_ARB_texture_non_power_of_two
		glTexImage2D( GL_TEXTURE_2D, 0,GL_LUMINANCE, MASK_SIZE, MASK_SIZE
			,0, GL_LUMINANCE, GL_UNSIGNED_BYTE, mask);

	}
	
	int make_tex(int size)// 2,4,8,16
	{
		char *tex;

		tex=(char *)malloc(size*size);
		int i,j;
		//for(i=0;i<size
	}

	int Update(Uint32 msec){ 
		float vp[3],v1[3];	
		int i,size;	
		extern Object *camera,space;
		size=particles.size();
		for(i=0;i<size;i++)
		{
			particles[i];
		}
		return 0;
		for(i=0;i<256*256;i++)
			if(mask[i]!=0)	mask[i]=mask[i]--;			
		//float g=system_getf(SDX_GRAVITY);

		glEnable(GL_TEXTURE_2D);
  		glBindTexture(GL_TEXTURE_2D,TexId);
		glBegin(GL_QUADS);
		glTexCoord2f(0,0); glVertex2f(0,0);
		glTexCoord2f(1,0); glVertex2f(w,0);
		glTexCoord2f(1,1); glVertex2f(w,h);
		glTexCoord2f(0,1); glVertex2f(0,h);
		glEnd();
	}
};


class Selector : public Object 
{	
	private:
		int token;
		Object *target;
		Object *viewer;
		List<Object*> list; // Index better !!!
	public:
	Selector():Object("Selector")
	{
		token=debug_token();
		target=0;

	}
	void setViewer(Object *obj)
	{
		viewer=obj;
	}
	
	int Update(Uint32 msec){ 
	
		for(int i=0;i<list.size();i++)
		{
			list[i]->flag_active=false;
			list[i]->flag_visiable=true;
		}
		list.clear();
				
		extern Object *camera;
		Line line;
		vectorCopy3f(line.onPoint,camera->position);
		vectorCopy3f(line.normal,camera->direction);
		line.length=30000;

		test_collision_line(&line);
		int size;	
		//debug_printf(token,"camera center(%f,%f,%f)",box->center[0],box->center[1],	box->center[2]);
		if(size=collisions.size())
		{	
			float d=9999999;
			Object *x=0;
			x=collisions[0];
		/*	for(int i=0;i<size;i++)
			{
				Object *m;
				m=collisions[i];	
				if(d>m->distance and  m->level<0)
				//and  m->level==TREE_LEAF)
				{
					d=m->distance;
					x=m;
				}	
			} */

			//for(int i=0;i<size;i++)
			if(x)
			{
				x->flag_active=1;
			//	x->flag_visiable=0;
				list.add(x);
			//	CSG(x);
				extern void CSG(Object *obj);
				/*
				x->collisions.clear();
				x->boundbox.test_collision(&space);
				int s=x->collisions.size();
				for(int i=0;i<s;i++)
				{
					Object *m;
					m=x->collisions[i];
				//	m->flag_active=2;
					list.add(m);
				} */
			}
			//for(int i=0;i<size;i++)
			//	CSG(list[i]);
		}
		debug_printf(token,"Selector: %d ",size);
	}

};

Frame *win_radar;
void init_octree(Object *map);
void Draw_Radar();
void draw_depth();
void test();
void game_setting()
{

	system_add(new FPS()); // fps counter 

	//Frame *main_frame=new Frame(50,50,400,400);
	//system_add(main_frame);
	
	win_radar=new Frame(0,0,200,200,Draw_Radar);
	win_radar->hide();
	system_add(win_radar);
	win_radar->move(-220,-220);
	
//	Frame *depth=new Frame(0,0,256,196,draw_depth);
//	system_add(depth);
//	depth->move(-220,300);
	Object *map=new Object("map");
//	Load_MAP_Hammer(map,"ground.map");
//	Load_MAP_Hammer(map,"sample1.map");
//	Load_RMF_Hammer(map,"de_dusto.rmf");
	Load_MAP_Hammer(map,"de_dust.map");
//	Load_MAP_Hammer(map,"ground.map");
	map_object=map;
	init_octree(map);//system_add(map);
	//Make_BitMap(map);

	if(1)
	{
		extern Object space;
		Object *terror=new Model_CS("terror.model");
		Object *death;
		terror->flag_bbox=1;
		death=system_find_object("death");
		if(death)
		{
			terror->setPosition(death->position); 
			//terror->force[1]+=0.01;
			system_add(terror);
		}
		else console_printf("can't find death_object in the map");
		//space.add(terror);
		Object *ai=new Tracer(terror);
		system_add(ai);
	}

	Object *viewer=new Viewer(); //	viewer->shape=0; // DEL??
	system_add(viewer);
	system_set_camera(viewer);
	//system_load_local_rc();  // configuration read !
	//printf("sizeof =%d \n",sizeof(Object*));
	
	Object *obj;
	obj=new Selector(); 

	system_add(obj);
	//obj=new Particler(); system_add(obj);
	//obj=new Navigator(); system_add(obj);
	//obj=new Optimizer(); system_add(obj);
	printf("game_setting() end\n");
}


int main(int argc, char** argv)
{  
   	int i,err;	
	//test();
	system_init();
	game_setting();
	system_loop();
	system_exit();
   	return 0;
}

void test_list()
{
	List<Object *> olist;
	olist.add(new Object("1"));
	olist.add(new Object("2"));
	olist.add(new Object("3"));
	olist.add(new Object("4"));
	olist.add(new Object("5"));
	olist.add(new Object("6"));
	olist.add(new Object("7"));

	for(olist.begin();olist.current()!=NULL;olist.next())
	{
		Object *o;
		o=olist.current();
		if(strcmp(o->name,"5")==0)
			olist.remove_current();
	}

	olist.reindex();

	for(olist.begin();olist.current()!=NULL;olist.next())
	{
		Object *o;
		o=olist.current();
		printf("oname=%s\n",o->name);
	}
	for(int i=0;i<olist.size();i++)
	{
		Object *o;
		o=olist[i];
		printf("oname=%s\n",o->name);
	}

}
