#ifndef SDX_MATH
#define SDX_MATH

#include <math.h>
#include "object/shape.h"

#define vector3f float
#define EPSILON 0.00001

#define RADIAN(x) 	( (float )(M_PI/180.0) * x)
#define DEGREE(x) 	( (float )(180.0 /M_PI )* x)


float vectorDistance( float a[3],float b[3]);

//float fast_sqrt(float f);
inline float fast_sqrt(float f)
{
	float result; // used to send out result
	asm(
		"mov %1,%%eax \n"
		"sub $0x3f800000,%%eax \n"
		"sar $1,%%eax \n"	
		"add $0x3f800000,%%eax \n"
		"mov %%eax,%0 \n":"=r"(result):"r"(f) );
/*
	asm("mov eax, f"
	"sub eax, 0x3f800000
	sar eax, 1
	add eax, 0x3f800000
	mov result, eax"
	); // end asm */
	return result;
} // end Fast_SquareRT

#define MATH_MACRO

#define M_vectorDotProduct3f(v1,v2)  v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2]

#ifdef MATH_MACRO
#define	vectorZero(vz) (vz[0]=0,vz[1]=0,vz[2]=0)
#define vectorDotProduct3f(v1,v2)  (v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2])
#define vectorDotProduct4f(v1,v2)  (v1[0]*v2[0] + v1[1]*v2[1] + v1[2]*v2[2] + v1[3]*v2[3])
#define vectorCopyf(o,in)		o[0] = in[0],	o[1] = in[1],o[2] = in[2]; 
#define vectorCopy3f(o,in)		o[0] = in[0],	o[1] = in[1],o[2] = in[2]; 
#define vectorCompare(a,b)		a[0] == b[0] && a[1] == b[1] && a[2] == b[2]
#define vectorLength(v)			sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])
#define vectorLength2(v)		fast_sqrt(v[0]*v[0]+v[1]*v[1]+v[2]*v[2])
//float	vectorLength(float v[3]);
#define vectorNormalize(v) \
{	float length= vectorLength(v); \
	if ( length < EPSILON) \
		v[0]=0,v[1]=0,v[2]=0;\
	else  v[0] /= length,v[1] /= length,v[2] /= length;}

//va[0]-=vb[0],va[1]-=vb[1],va[2]=vb[2] 
#define vectorSubtractM(va,vb) 		va[0] = va[0]-vb[0]; va[1] = va[1]-vb[1]; va[2] = va[2]-vb[2]; 
#define vectorSubtract2f(va,vb) 	va[0] = va[0]-vb[0]; va[1] = va[1]-vb[1]; va[2] = va[2]-vb[2]; 
#define vectorSubtract(vo,va,vb) 	vo[0] = va[0]-vb[0]; vo[1] = va[1]-vb[1]; vo[2] = va[2]-vb[2]; 

#define FAST_ROUND(fx) int(fx + 0.5f)

#define matrixMultvector4f(mat, v,out) \
	out[0] = vectorDotProduct4f(mat[0],v); \
	out[1] = vectorDotProduct4f(mat[1],v); \
	out[2] = vectorDotProduct4f(mat[2],v); \
	out[3] = vectorDotProduct4f(mat[3],v); \

#define vectorDotProduct3fx2(v1,v2,v3,dot) \
	asm (	"movups %1,%%xmm0;" \
			"mulps %2,%%xmm0;" \
			"movups %%xmm0,%0;" \
			:"=m"(v3[0]):"m"(v1[0]),"m"(v2[0]));	dot=v3[0]+v3[1]+v3[2]; 

#endif

#ifndef MATH_MACRO

#endif

float 	vectorAngle(float v1[3],float v2[3]);

void	vertexCopy(Vertex *vr,vector v[3]);
void	vertexCopy(Vertex *vo,Vertex *vin);

void	vectorPrint(vector v[3]);
void	vectorSet  (vector *v, float x,float y, float z);
void	vectorAdd (vector3f out[3], vector3f vb[3]);
void	vectorAdd (vector3f out[3], vector3f va[3], vector3f vb[3]);
//void	vectorSubtract (vector3f *a, vector3f *b);
//void	vectorSubtract (vector3f out[3],vector3f a[3], vector3f b[3]);
void	vectorScale (vector3f v[3], float scale); 
//float	vectorDotProduct4f(float v1[4], float v2[4]);
float	vectorDotProduct3fx(float v1[4], float v2[4]);
void	vectorCrossProduct (float out[3],float v1[3], float  v2[3]);
int	vectorNormalizeN(float v[3]);

//void	vectorCopy (float out[3],float in[3]);
void	vectorCopy (Vertex *v,float in[3]);
void	vectorCopy (float out[3],Vertex *in);
void	vectorCopy(int out[3],float in[3]);

void	vectors2normal(float v1[3], float v2[3],float v3[3], float out[3]);
void	vectorRotate(float v[3],float angle,float  up[3]);


void	matrixTranslate(float mat[4][4],float v[3]);
void	matrixPrint(float mat[4][4]);
void	matrixZero(float mat[4][4]);
void	matrixCopy(float out[4][4],float in[4][4]);
void	matrixTranspose(float mat[4][4]);
void	matrixAngles (float angles[3], float mat[4][4] );
void	matrixMult(float in1[4][4],  float in2[4][4], float out[4][4]);
void	matrixMultvector(float mat[4][4], float v[3],  float out[3]);
//void	matrixMultvector4f(float mat[4][4], float v[4],  float out[4]);
void	matrixIMultvector( float m[4][4],float in1[3], float out[3]);

int		test_intersect_triangle_line4(float *verts[3],Line *line1,float *distance);
int		test_intersect_square_line(float *verts[3],Line *line,float *distance);


//#define VECTOR(x) (float *)x
//#define vec3_t float 

//void 		VectorSwap(Vector *v1, Vector *v2);
//void 		VectorSetZero (Vector *v1);
//inline   void 	VectorSetVector(Vector *v1, Vector *v2);
//void    	VectorSet(Vector *v1, float,float,float);

// DEL #define VertexCopy VectorCopy
//inline	void 	VectorCopy (Vertex *v1,Vertex *v2);
//inline	void 	VectorScale (Vector *v1, float scale) ;
//inline	void 	VectorAdd  (Vector *v1, Vector  *v2);
//inline	void 	VectorSubtract  (Vector *v1, Vector  *v2);
//inline 	int 	VectorNormalize(Vector *V); 
//inline	float 	VectorDotProduct (Vector *V1, Vector *V2);
//inline  void 	VectorCrossProduct (Vector *r,Vector *v1, Vector  *v2);

//float 		VectorLength(Vector *v);

//inline	void 	LineSetVector(Line *l,Vector *v1, Vector *v2);
//void 		LineCopy(Line *L1,Line *L2);
/*
void 		vectorSet  (vector *v, float x,float y, float z); 	// OK
void 		vectorCopy (vec3_t out[3],vec3_t in[3]);  		// OK
void 		vectorScale (vec3_t v[3], float scale); 		// OK
inline void 	vectorAdd (vec3_t va[3], vec3_t vb[3]); 		// OK
void 		vectorAddV3 (float a[3],float b[3],vec3_t out[3]) ;
void 		vectorSubtract (vec3_t va[3], vec3_t vb[3]); 		// OK
	void	vectorSubtractV3 (vec3_t *va, vec3_t *vb,vec3_t *out) ; // OK
inline float 	vectorDotProduct (vec3_t *v1, vec3_t *v2);		
inline float	vectorLength(vector v[3]);
void 		vectorNormalize(float v[3]); 
float 		vectorAngle(vector *v1,vector *v2);
void 		vectorRotate(float v[3],float  up[3] ,float angle );


inline void 	vectorCrossProduct (float cross[3],float v1[3], float  v2[3]);

inline  void 	Vector2vector(float v[3],Vector *V) ;
inline  void 	vector2Vector(Vector *V,float v[3]) ;
void 		Matrix_transposeV2( GLfloat to[16], const GLfloat from[16] );


void 		PlaneSetNormalVertex(Plane *P,Vector *N,Vector *v);

void 		Three2Normal(float v1[3], float v2[3],float v3[3], float out[3]) ;
int 		VectorIntersction3Plane(Vector *V,Plane *p1,Plane *p2,Plane *p3);
int  		VectorIntersectionLinePlane(Vector *v,Line *l,Plane *p);
	
inline int      test_intersect_triangle (Triangle *tri,float dir[3],float orig[3],float p[3],float *d);
inline int 	test_intersect_triangle_line (Triangle *tri,line *line1,float p[3],float *distance);
inline int 	test_intersect_triangle_line2(Triangle *tri,line *line1,float *distance);
inline int 	test_intersect_triangle_line3(Triangle *tri,line *line1,float *distance);
inline int 	test_intersect_triangle_line4(float *verts[3],line *line1,float *distance);

void 		make_triangle_norm(Triangle *tri);
void 		AngleMatrix (float *angle, float (*matrix)[4] );
void 		AngleIMatrix (float angles[3], float matrix[4][4] );
void 		R_ConcatTransforms (float in1[4][4], float in2[4][4], float out[4][4]);
void 		Matrix_Add ( float in2[4][4],float in1[4][4]);
void 		Matrix_vectorRotate ( float in2[4][4],float in1[3],  float  out[3]);
void 		Matrix_vectorMult( float matrix[4][4],float in1[3]);
void 		Matrix_vectorMultV3 ( float matirx[4][4],float in_v[3],  float  out[3]);
void 		Matrix_vectorIRotate ( float in2[4][4],float in1[3],  float  out[3]);
void 		Matrix_vectorTransform (float in2[4][4], vec3_t in1[3],  vec3_t out[3]);
void 		Matrix_transpose(float matirx[4][4]);
void 		Matrix_angles(float m[16],float angles[3]);
void 		AngleXYZMatrix (float *angles, float (*matrix)[4] );

//ӽ 
void 	Matrix_y(float m[16],float angle);
void 	Matrix_x(float m[16],float angle);

float 		get_distance_v2v(float A[3],float B[3]);

*/

#endif

