#ifndef SYSTEM_H
#define SYSTEM_H

#include <SDL/SDL.h>
//#define GL_GLEXT_PROTOTYPES 1
#include <SDL/SDL_opengl.h>
//#include <SDL/SDL_net.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#define UNIT		100	// 1 meter
#define ONEMETER	100

#define vector	float
#define	vector3f float
//typedef float vector; 
//typedef float vector3f; 

#define SYSTEM_WIDTH  1
#define SYSTEM_HEIGHT 2

#define TREE_LEAF		3
#define FOV_FULLIN		64
#define FOV_PARTIALIN   	128

#include "object/object.h"
#include "object/shape.h"
#include "ui/widgets.h"
#include "misc.h"
#include "math.h"
#include "list.h"

void system_exit();
void system_init();
void system_loop();
void system_input(SDL_Event *event);
void system_input_disable();
void system_insert(Object *obj);
void system_add(Widget *w);
void system_add(Object *o);
void system_set_camera(Object *obj);
void system_mouse_center();
int  system_checkMouseCenter(int x,int y);
int  system_geti(int id);

void draw_axes();

Object *system_find_object(const char *name);
void console_printf(const char *fmt,...);
void debug_printf(int token,const char *fmt,...);
int  debug_token();
void debugclear();

#ifdef WIN32
	#define random rand

#endif

class PlaneViewer : public Object 
{	
	private:
		List<Line*>		lines;
		List<Plane*>	planes;
		List<Vertex*>	vertexs;
		Object*			PlaneObject;
	public:
	PlaneViewer();
	void addPlane(Plane *p,int idx);
	void addVertex(Vertex *p);
	void addLine(Line *l);
	void event();
	virtual int Update(Uint32 msec);
};

#define CHECK_OPENGL_ERROR( cmd ) cmd; \
{  GLenum error; \
	while ( (error = glGetError()) != GL_NO_ERROR)  \
	printf( "[%s:%d]%s : failed with error= %s\n", __FILE__, __LINE__, #cmd, gluErrorString(error) ); }

#define CHECK_OPENGL() \
{ GLenum error; \
	while ( (error = glGetError()) != GL_NO_ERROR)  \
	printf( "[%s:%d] xxx failed with error %s\n", __FILE__, \
	__LINE__,  gluErrorString(error) ); \
}



#endif //SYSTEM_H
