
#ifndef WIDGETS_H
#define WIDGETS_H

#include <SDL/SDL.h>
#include "event.h"
#include "../list.h"

int drawString(int x,int y,const char *str); // ui.h
void setWindowHeight(int h);
void setColor(char r,char g,char b,char a);
void drawRectLine(int x,int y,int w,int h);
void drawRect(int x,int y,int w,int h);


class Widget {
	public:
	//Widget::Widget (){}
	Widget()
	{
		parent=NULL;
		move(0,0);
		resize(0,0);
		visible=1;
	};
	
	~Widget(){};
	void addChild( Widget *w);
	void add( Widget *w);
	void setParent(Widget *parent);
	void resize(int w, int h);
	void move(int x, int y);
	void registerEvent(int eventType,void *handler);
	void hide(){ visible=0;}
	void show(){ visible=1;}
	int handleEvent(SDL_Event *event);
	void drawLine(int x1,int y1,int x2,int y2)
	{
		//"""Draws a line"""
		glBegin(GL_LINES);
		glVertex2i(x1, parent->height-y1);
		glVertex2i(x2, parent->height-y2);
		glEnd();
	}	

	virtual	void Draw(){};
	
	int x,y;
	int width;
	int height;
	int visible;

	Widget *parent;
	Index<Widget*> children;
	
//	protected:
	private:
	char name[32];	
	List<UI_Event*> events;
	int mouse_in;
};

class Frame : public Widget 
{	
	public:
	Frame(int x, int y,int w,int h);
	Frame(int x, int y,int w,int h,void (*draw)());
	void Draw();
	private:
	void (*draw_func)();
};

class Console : public Widget 
{	
	public:
	Console(int x, int y,int w,int h);
	void Draw();
	void clear();
	void debugclear();
	void printf(char *fmt,...);
	void addline(char *str);
	int debug_token();
	void debug(int t,char *str);
	private:
	List_string lines;
	List_string debugs;
};

class Debug : public Widget 
{	
	public:
	Debug(int x, int y,int w,int h);
	void Draw();
	void clear();
//	void printf(char *fmt,...);
	void add(char *str);
	private:
	List_string lines;
};


#endif 
